/*
 * Copyright © 2015 Red Hat, Inc
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *       Alexander Larsson <alexl@redhat.com>
 */

#if !defined(__KAIMING_H_INSIDE__) && !defined(KAIMING_COMPILATION)
#error "Only <kaiming.h> can be included directly."
#endif

#ifndef __KAIMING_INSTALLATION_H__
#define __KAIMING_INSTALLATION_H__

typedef struct _KaimingInstallation KaimingInstallation;

#include <gio/gio.h>
#include <kaiming-installed-ref.h>
#include <kaiming-instance.h>
#include <kaiming-remote.h>

G_BEGIN_DECLS

#define KAIMING_TYPE_INSTALLATION kaiming_installation_get_type ()
#define KAIMING_INSTALLATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), KAIMING_TYPE_INSTALLATION, KaimingInstallation))
#define KAIMING_IS_INSTALLATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KAIMING_TYPE_INSTALLATION))

KAIMING_EXTERN GType kaiming_installation_get_type (void);

struct _KaimingInstallation
{
  GObject parent;
};

typedef struct
{
  GObjectClass parent_class;
} KaimingInstallationClass;

/**
 * KaimingUpdateFlags:
 * @KAIMING_UPDATE_FLAGS_NONE: Fetch remote builds and install the latest one (default)
 * @KAIMING_UPDATE_FLAGS_NO_DEPLOY: Don't install any new builds that might be fetched
 * @KAIMING_UPDATE_FLAGS_NO_PULL: Don't try to fetch new builds from the remote repo
 * @KAIMING_UPDATE_FLAGS_NO_STATIC_DELTAS: Don't use static deltas when pulling
 * @KAIMING_UPDATE_FLAGS_NO_PRUNE: Don't prune the local OSTree repository after updating (Since: 0.11.8)
 * @KAIMING_UPDATE_FLAGS_NO_TRIGGERS: Don't call triggers after updating. If used,
 * the caller must later call kaiming_installation_run_triggers() to update
 * the exported files. (Since: 1.0.3)
 *
 * Flags to alter the behavior of kaiming_installation_update().
 */
typedef enum {
  KAIMING_UPDATE_FLAGS_NONE             = 0,
  KAIMING_UPDATE_FLAGS_NO_DEPLOY        = (1 << 0),
  KAIMING_UPDATE_FLAGS_NO_PULL          = (1 << 1),
  KAIMING_UPDATE_FLAGS_NO_STATIC_DELTAS = (1 << 2),
  KAIMING_UPDATE_FLAGS_NO_PRUNE         = (1 << 3),
  KAIMING_UPDATE_FLAGS_NO_TRIGGERS      = (1 << 4),
} KaimingUpdateFlags;

/**
 * KaimingInstallFlags:
 * @KAIMING_INSTALL_FLAGS_NONE: Default
 * @KAIMING_INSTALL_FLAGS_NO_STATIC_DELTAS: Don't use static deltas when pulling
 * @KAIMING_INSTALL_FLAGS_NO_DEPLOY: Don't install any new builds that might be fetched
 * @KAIMING_INSTALL_FLAGS_NO_PULL: Don't try to fetch new builds from the remote repo
 * @KAIMING_INSTALL_FLAGS_NO_TRIGGERS: Don't call triggers after installing. If used,
 * the caller must later call kaiming_installation_run_triggers() to update
 * the exported files. (Since: 1.0.3)
 *
 * Flags to alter the behavior of kaiming_installation_install_full().
 */
typedef enum {
  KAIMING_INSTALL_FLAGS_NONE             = 0,
  KAIMING_INSTALL_FLAGS_NO_STATIC_DELTAS = (1 << 0),
  KAIMING_INSTALL_FLAGS_NO_DEPLOY        = (1 << 2),
  KAIMING_INSTALL_FLAGS_NO_PULL          = (1 << 3),
  KAIMING_INSTALL_FLAGS_NO_TRIGGERS      = (1 << 4),
} KaimingInstallFlags;

/**
 * KaimingUninstallFlags:
 * @KAIMING_UNINSTALL_FLAGS_NONE: Default
 * @KAIMING_UNINSTALL_FLAGS_NO_PRUNE: Don't prune the local OSTree repository after uninstalling
 * @KAIMING_UNINSTALL_FLAGS_NO_TRIGGERS: Don't call triggers after uninstalling. If used,
 * the caller must later call kaiming_installation_run_triggers() to update
 * the exported file. (Since: 1.0.3)
 *
 * Flags to alter the behavior of kaiming_installation_uninstall_full().
 *
 * Since: 0.11.8
 */
typedef enum {
  KAIMING_UNINSTALL_FLAGS_NONE            = 0,
  KAIMING_UNINSTALL_FLAGS_NO_PRUNE        = (1 << 0),
  KAIMING_UNINSTALL_FLAGS_NO_TRIGGERS     = (1 << 1),
} KaimingUninstallFlags;

/**
 * KaimingLaunchFlags:
 * @KAIMING_LAUNCH_FLAGS_NONE: Default
 * @KAIMING_LAUNCH_FLAGS_DO_NOT_REAP: Do not reap the child. Use this if you want to wait
 * for the child with g_child_watch_add(). (Since: 1.1)
 *
 * Flags to alter the behavior of kaiming_installation_launch_full().
 */
typedef enum {
  KAIMING_LAUNCH_FLAGS_NONE        = 0,
  KAIMING_LAUNCH_FLAGS_DO_NOT_REAP = (1 << 0),
} KaimingLaunchFlags;

/**
 * KaimingQueryFlags:
 * @KAIMING_QUERY_FLAGS_NONE: Default
 * @KAIMING_QUERY_FLAGS_ONLY_CACHED: Don't do any network i/o, but only return cached data.
 * This can return stale data, or a #KAIMING_ERROR_NOT_CACHED error, however it is a
 * lot more efficient if you're doing many requests.
 * @KAIMING_QUERY_FLAGS_ONLY_SIDELOADED: Only list refs available from sideload
 * repos; see kaiming(1). (Since: 1.7)
 * @KAIMING_QUERY_FLAGS_ALL_ARCHES: Include refs from all arches, not just the primary ones. (Since: 1.11.2)
 *
 * Flags to alter the behavior of e.g kaiming_installation_list_remote_refs_sync_full().
 *
 * Since: 1.3.3
 */
typedef enum {
  KAIMING_QUERY_FLAGS_NONE        = 0,
  KAIMING_QUERY_FLAGS_ONLY_CACHED = (1 << 0),
  KAIMING_QUERY_FLAGS_ONLY_SIDELOADED = (1 << 1),
  KAIMING_QUERY_FLAGS_ALL_ARCHES = (1 << 2),
} KaimingQueryFlags;

/**
 * KaimingStorageType:
 * @KAIMING_STORAGE_TYPE_DEFAULT: default
 * @KAIMING_STORAGE_TYPE_HARD_DISK: installation is on a hard disk
 * @KAIMING_STORAGE_TYPE_SDCARD: installation is on a SD card
 * @KAIMING_STORAGE_TYPE_MMC: installation is on an MMC
 * @KAIMING_STORAGE_TYPE_NETWORK: installation is on the network
 *
 * Information about the storage of an installation.
 *
 * Since: 0.6.15
 */
typedef enum {
  KAIMING_STORAGE_TYPE_DEFAULT = 0,
  KAIMING_STORAGE_TYPE_HARD_DISK,
  KAIMING_STORAGE_TYPE_SDCARD,
  KAIMING_STORAGE_TYPE_MMC,
  KAIMING_STORAGE_TYPE_NETWORK,
} KaimingStorageType;


#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
G_DEFINE_AUTOPTR_CLEANUP_FUNC (KaimingInstallation, g_object_unref)
#endif

KAIMING_EXTERN const char  *kaiming_get_default_arch (void);

KAIMING_EXTERN const char *const *kaiming_get_supported_arches (void);

KAIMING_EXTERN GPtrArray *kaiming_get_system_installations (GCancellable *cancellable,
                                                            GError      **error);
KAIMING_EXTERN KaimingInstallation *kaiming_installation_new_system (GCancellable *cancellable,
                                                                     GError      **error);
KAIMING_EXTERN KaimingInstallation *kaiming_installation_new_system_with_id (const char   *id,
                                                                             GCancellable *cancellable,
                                                                             GError      **error);
KAIMING_EXTERN KaimingInstallation *kaiming_installation_new_user (GCancellable *cancellable,
                                                                   GError      **error);
KAIMING_EXTERN KaimingInstallation *kaiming_installation_new_for_path (GFile        *path,
                                                                       gboolean      user,
                                                                       GCancellable *cancellable,
                                                                       GError      **error);
KAIMING_EXTERN void kaiming_installation_set_no_interaction (KaimingInstallation *self,
                                                             gboolean             no_interaction);
KAIMING_EXTERN gboolean kaiming_installation_get_no_interaction (KaimingInstallation *self);

/**
 * KaimingProgressCallback:
 * @status: A status string, suitable for display
 * @progress: percentage of completion
 * @estimating: whether @progress is just an estimate
 * @user_data: User data passed to the caller
 *
 * The progress callback is called repeatedly during long-running operations
 * such as installations or updates, and can be used to update progress information
 * in a user interface.
 *
 * The callback occurs in the thread-default context of the caller.
 */
typedef void (*KaimingProgressCallback)(const char *status,
                                        guint       progress,
                                        gboolean    estimating,
                                        gpointer    user_data);

KAIMING_EXTERN gboolean             kaiming_installation_drop_caches (KaimingInstallation *self,
                                                                      GCancellable        *cancellable,
                                                                      GError             **error);
KAIMING_EXTERN gboolean             kaiming_installation_get_is_user (KaimingInstallation *self);
KAIMING_EXTERN GFile               *kaiming_installation_get_path (KaimingInstallation *self);
KAIMING_EXTERN const char          *kaiming_installation_get_id (KaimingInstallation *self);
KAIMING_EXTERN const char          *kaiming_installation_get_display_name (KaimingInstallation *self);
KAIMING_EXTERN gint                 kaiming_installation_get_priority (KaimingInstallation *self);
KAIMING_EXTERN KaimingStorageType   kaiming_installation_get_storage_type (KaimingInstallation *self);
KAIMING_EXTERN gboolean             kaiming_installation_launch (KaimingInstallation *self,
                                                                 const char          *name,
                                                                 const char          *arch,
                                                                 const char          *branch,
                                                                 const char          *commit,
                                                                 GCancellable        *cancellable,
                                                                 GError             **error);
KAIMING_EXTERN gboolean             kaiming_installation_launch_full (KaimingInstallation *self,
                                                                      KaimingLaunchFlags   flags,
                                                                      const char          *name,
                                                                      const char          *arch,
                                                                      const char          *branch,
                                                                      const char          *commit,
                                                                      KaimingInstance    **instance_out,
                                                                      GCancellable        *cancellable,
                                                                      GError             **error);
KAIMING_EXTERN GFileMonitor        *kaiming_installation_create_monitor (KaimingInstallation *self,
                                                                         GCancellable        *cancellable,
                                                                         GError             **error);
KAIMING_EXTERN GPtrArray           *kaiming_installation_list_installed_refs (KaimingInstallation *self,
                                                                              GCancellable        *cancellable,
                                                                              GError             **error);
KAIMING_EXTERN GPtrArray           *kaiming_installation_list_installed_refs_by_kind (KaimingInstallation *self,
                                                                                      KaimingRefKind       kind,
                                                                                      GCancellable        *cancellable,
                                                                                      GError             **error);
KAIMING_EXTERN GPtrArray           *kaiming_installation_list_installed_refs_for_update (KaimingInstallation *self,
                                                                                         GCancellable        *cancellable,
                                                                                         GError             **error);
KAIMING_EXTERN GPtrArray           *kaiming_installation_list_unused_refs (KaimingInstallation *self,
                                                                           const char          *arch,
                                                                           GCancellable        *cancellable,
                                                                           GError             **error);
KAIMING_EXTERN GPtrArray           *kaiming_installation_list_unused_refs_with_options (KaimingInstallation *self,
                                                                                        const char          *arch,
                                                                                        GHashTable          *metadata_injection,
                                                                                        GVariant            *options,
                                                                                        GCancellable        *cancellable,
                                                                                        GError             **error);
KAIMING_EXTERN GPtrArray           *kaiming_installation_list_pinned_refs (KaimingInstallation *self,
                                                                           const char          *arch,
                                                                           GCancellable        *cancellable,
                                                                           GError             **error);
KAIMING_EXTERN KaimingInstalledRef * kaiming_installation_get_installed_ref (KaimingInstallation *self,
                                                                             KaimingRefKind       kind,
                                                                             const char          *name,
                                                                             const char          *arch,
                                                                             const char          *branch,
                                                                             GCancellable        *cancellable,
                                                                             GError             **error);
KAIMING_EXTERN KaimingInstalledRef * kaiming_installation_get_current_installed_app (KaimingInstallation *self,
                                                                                     const char          *name,
                                                                                     GCancellable        *cancellable,
                                                                                     GError             **error);

KAIMING_EXTERN GPtrArray           *kaiming_installation_list_remotes (KaimingInstallation *self,
                                                                       GCancellable        *cancellable,
                                                                       GError             **error);
KAIMING_EXTERN GPtrArray           *kaiming_installation_list_remotes_by_type (KaimingInstallation    *self,
                                                                               const KaimingRemoteType types[],
                                                                               gsize                   num_types,
                                                                               GCancellable           *cancellable,
                                                                               GError                **error);
KAIMING_EXTERN KaimingRemote        *kaiming_installation_get_remote_by_name (KaimingInstallation *self,
                                                                              const gchar         *name,
                                                                              GCancellable        *cancellable,
                                                                              GError             **error);
KAIMING_EXTERN gboolean              kaiming_installation_add_remote (KaimingInstallation *self,
                                                                      KaimingRemote       *remote,
                                                                      gboolean             if_needed,
                                                                      GCancellable        *cancellable,
                                                                      GError             **error);
KAIMING_EXTERN gboolean              kaiming_installation_modify_remote (KaimingInstallation *self,
                                                                         KaimingRemote       *remote,
                                                                         GCancellable        *cancellable,
                                                                         GError             **error);
KAIMING_EXTERN gboolean              kaiming_installation_remove_remote (KaimingInstallation *self,
                                                                         const char          *name,
                                                                         GCancellable        *cancellable,
                                                                         GError             **error);
KAIMING_EXTERN gboolean              kaiming_installation_update_remote_sync (KaimingInstallation *self,
                                                                              const char          *name,
                                                                              GCancellable        *cancellable,
                                                                              GError             **error);
KAIMING_EXTERN gboolean              kaiming_installation_set_config_sync (KaimingInstallation *self,
                                                                           const char          *key,
                                                                           const char          *value,
                                                                           GCancellable        *cancellable,
                                                                           GError             **error);
KAIMING_EXTERN gboolean              kaiming_installation_get_min_free_space_bytes (KaimingInstallation *self,
                                                                                    guint64             *out_bytes,
                                                                                    GError             **error);
KAIMING_EXTERN char *                kaiming_installation_get_config (KaimingInstallation *self,
                                                                      const char          *key,
                                                                      GCancellable        *cancellable,
                                                                      GError             **error);
KAIMING_EXTERN char **               kaiming_installation_get_default_languages (KaimingInstallation  *self,
                                                                                 GError              **error);
KAIMING_EXTERN char **               kaiming_installation_get_default_locales (KaimingInstallation  *self,
                                                                               GError              **error);
KAIMING_EXTERN char *              kaiming_installation_load_app_overrides (KaimingInstallation *self,
                                                                            const char          *app_id,
                                                                            GCancellable        *cancellable,
                                                                            GError             **error);
G_GNUC_DEPRECATED_FOR(kaiming_transaction_add_install)
KAIMING_EXTERN KaimingInstalledRef * kaiming_installation_install (KaimingInstallation    *self,
                                                                   const char             *remote_name,
                                                                   KaimingRefKind          kind,
                                                                   const char             *name,
                                                                   const char             *arch,
                                                                   const char             *branch,
                                                                   KaimingProgressCallback progress,
                                                                   gpointer                progress_data,
                                                                   GCancellable           *cancellable,
                                                                   GError                **error);
G_GNUC_DEPRECATED_FOR(kaiming_transaction_add_install)
KAIMING_EXTERN KaimingInstalledRef * kaiming_installation_install_full (KaimingInstallation    *self,
                                                                        KaimingInstallFlags     flags,
                                                                        const char             *remote_name,
                                                                        KaimingRefKind          kind,
                                                                        const char             *name,
                                                                        const char             *arch,
                                                                        const char             *branch,
                                                                        const char * const     *subpaths,
                                                                        KaimingProgressCallback progress,
                                                                        gpointer                progress_data,
                                                                        GCancellable           *cancellable,
                                                                        GError                **error);
G_GNUC_DEPRECATED_FOR(kaiming_transaction_add_update)
KAIMING_EXTERN KaimingInstalledRef * kaiming_installation_update (KaimingInstallation    *self,
                                                                  KaimingUpdateFlags      flags,
                                                                  KaimingRefKind          kind,
                                                                  const char             *name,
                                                                  const char             *arch,
                                                                  const char             *branch,
                                                                  KaimingProgressCallback progress,
                                                                  gpointer                progress_data,
                                                                  GCancellable           *cancellable,
                                                                  GError                **error);
G_GNUC_DEPRECATED_FOR(kaiming_transaction_add_update)
KAIMING_EXTERN KaimingInstalledRef * kaiming_installation_update_full (KaimingInstallation    *self,
                                                                       KaimingUpdateFlags      flags,
                                                                       KaimingRefKind          kind,
                                                                       const char             *name,
                                                                       const char             *arch,
                                                                       const char             *branch,
                                                                       const char * const     *subpaths,
                                                                       KaimingProgressCallback progress,
                                                                       gpointer                progress_data,
                                                                       GCancellable           *cancellable,
                                                                       GError                **error);
G_GNUC_DEPRECATED_FOR(kaiming_transaction_add_install_bundle)
KAIMING_EXTERN KaimingInstalledRef * kaiming_installation_install_bundle (KaimingInstallation    *self,
                                                                          GFile                  *file,
                                                                          KaimingProgressCallback progress,
                                                                          gpointer                progress_data,
                                                                          GCancellable           *cancellable,
                                                                          GError                **error);
G_GNUC_DEPRECATED_FOR(kaiming_transaction_add_install_kaimingref)
KAIMING_EXTERN KaimingRemoteRef *   kaiming_installation_install_ref_file (KaimingInstallation *self,
                                                                           GBytes              *ref_file_data,
                                                                           GCancellable        *cancellable,
                                                                           GError             **error);
G_GNUC_DEPRECATED_FOR(kaiming_transaction_add_uninstall)
KAIMING_EXTERN gboolean             kaiming_installation_uninstall (KaimingInstallation    *self,
                                                                    KaimingRefKind          kind,
                                                                    const char             *name,
                                                                    const char             *arch,
                                                                    const char             *branch,
                                                                    KaimingProgressCallback progress,
                                                                    gpointer                progress_data,
                                                                    GCancellable           *cancellable,
                                                                    GError                **error);

G_GNUC_DEPRECATED_FOR(kaiming_transaction_add_uninstall)
KAIMING_EXTERN gboolean             kaiming_installation_uninstall_full (KaimingInstallation    *self,
                                                                         KaimingUninstallFlags   flags,
                                                                         KaimingRefKind          kind,
                                                                         const char             *name,
                                                                         const char             *arch,
                                                                         const char             *branch,
                                                                         KaimingProgressCallback progress,
                                                                         gpointer                progress_data,
                                                                         GCancellable           *cancellable,
                                                                         GError                **error);

KAIMING_EXTERN gboolean          kaiming_installation_fetch_remote_size_sync (KaimingInstallation *self,
                                                                              const char          *remote_name,
                                                                              KaimingRef          *ref,
                                                                              guint64             *download_size,
                                                                              guint64             *installed_size,
                                                                              GCancellable        *cancellable,
                                                                              GError             **error);
KAIMING_EXTERN GBytes        *   kaiming_installation_fetch_remote_metadata_sync (KaimingInstallation *self,
                                                                                  const char          *remote_name,
                                                                                  KaimingRef          *ref,
                                                                                  GCancellable        *cancellable,
                                                                                  GError             **error);
KAIMING_EXTERN GPtrArray    *    kaiming_installation_list_remote_refs_sync (KaimingInstallation *self,
                                                                             const char          *remote_or_uri,
                                                                             GCancellable        *cancellable,
                                                                             GError             **error);
KAIMING_EXTERN GPtrArray    *    kaiming_installation_list_remote_refs_sync_full (KaimingInstallation *self,
                                                                                  const char          *remote_or_uri,
                                                                                  KaimingQueryFlags    flags,
                                                                                  GCancellable        *cancellable,
                                                                                  GError             **error);
KAIMING_EXTERN KaimingRemoteRef  *kaiming_installation_fetch_remote_ref_sync (KaimingInstallation *self,
                                                                              const char          *remote_name,
                                                                              KaimingRefKind       kind,
                                                                              const char          *name,
                                                                              const char          *arch,
                                                                              const char          *branch,
                                                                              GCancellable        *cancellable,
                                                                              GError             **error);
KAIMING_EXTERN KaimingRemoteRef  *kaiming_installation_fetch_remote_ref_sync_full (KaimingInstallation *self,
                                                                                   const char          *remote_name,
                                                                                   KaimingRefKind       kind,
                                                                                   const char          *name,
                                                                                   const char          *arch,
                                                                                   const char          *branch,
                                                                                   KaimingQueryFlags    flags,
                                                                                   GCancellable        *cancellable,
                                                                                   GError             **error);
KAIMING_EXTERN gboolean          kaiming_installation_update_appstream_sync (KaimingInstallation *self,
                                                                             const char          *remote_name,
                                                                             const char          *arch,
                                                                             gboolean            *out_changed,
                                                                             GCancellable        *cancellable,
                                                                             GError             **error);
KAIMING_EXTERN gboolean          kaiming_installation_update_appstream_full_sync (KaimingInstallation    *self,
                                                                                  const char             *remote_name,
                                                                                  const char             *arch,
                                                                                  KaimingProgressCallback progress,
                                                                                  gpointer                progress_data,
                                                                                  gboolean               *out_changed,
                                                                                  GCancellable           *cancellable,
                                                                                  GError                **error);
KAIMING_EXTERN GPtrArray    *    kaiming_installation_list_remote_related_refs_sync (KaimingInstallation *self,
                                                                                     const char          *remote_name,
                                                                                     const char          *ref,
                                                                                     GCancellable        *cancellable,
                                                                                     GError             **error);
KAIMING_EXTERN GPtrArray    *    kaiming_installation_list_remote_related_refs_for_installed_sync (KaimingInstallation *self,
                                                                                                   const char          *remote_name,
                                                                                                   const char          *ref,
                                                                                                   GCancellable        *cancellable,
                                                                                                   GError             **error);
KAIMING_EXTERN GPtrArray    *    kaiming_installation_list_installed_related_refs_sync (KaimingInstallation *self,
                                                                                        const char          *remote_name,
                                                                                        const char          *ref,
                                                                                        GCancellable        *cancellable,
                                                                                        GError             **error);

KAIMING_EXTERN gboolean          kaiming_installation_remove_local_ref_sync (KaimingInstallation *self,
                                                                             const char          *remote_name,
                                                                             const char          *ref,
                                                                             GCancellable        *cancellable,
                                                                             GError             **error);
KAIMING_EXTERN gboolean          kaiming_installation_cleanup_local_refs_sync (KaimingInstallation *self,
                                                                               GCancellable        *cancellable,
                                                                               GError             **error);
KAIMING_EXTERN gboolean          kaiming_installation_prune_local_repo (KaimingInstallation *self,
                                                                        GCancellable        *cancellable,
                                                                        GError             **error);
KAIMING_EXTERN gboolean          kaiming_installation_run_triggers (KaimingInstallation *self,
                                                                    GCancellable        *cancellable,
                                                                    GError             **error);

G_END_DECLS

#endif /* __KAIMING_INSTALLATION_H__ */
