/*
 * Copyright © 2016 Red Hat, Inc
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *       Alexander Larsson <alexl@redhat.com>
 */

#if !defined(__KAIMING_H_INSIDE__) && !defined(KAIMING_COMPILATION)
#error "Only <kaiming.h> can be included directly."
#endif

#ifndef __KAIMING_TRANSACTION_H__
#define __KAIMING_TRANSACTION_H__

#include <gio/gio.h>
#include <kaiming-installation.h>

G_BEGIN_DECLS

#define KAIMING_TYPE_TRANSACTION kaiming_transaction_get_type ()
#define KAIMING_TYPE_TRANSACTION_PROGRESS kaiming_transaction_progress_get_type ()
#define KAIMING_TYPE_TRANSACTION_OPERATION kaiming_transaction_operation_get_type ()

/**
 * KaimingTransactionOperationType
 * @KAIMING_TRANSACTION_OPERATION_INSTALL: Install a ref from a remote
 * @KAIMING_TRANSACTION_OPERATION_UPDATE: Update an installed ref
 * @KAIMING_TRANSACTION_OPERATION_INSTALL_BUNDLE: Install a bundle from a file
 * @KAIMING_TRANSACTION_OPERATION_UNINSTALL: Uninstall a ref
 * @KAIMING_TRANSACTION_OPERATION_LAST_TYPE: The (currently) last operation type
 *
 * The type of a #KaimingTransactionOperation.
 */
typedef enum {
  KAIMING_TRANSACTION_OPERATION_INSTALL,
  KAIMING_TRANSACTION_OPERATION_UPDATE,
  KAIMING_TRANSACTION_OPERATION_INSTALL_BUNDLE,
  KAIMING_TRANSACTION_OPERATION_UNINSTALL,
  KAIMING_TRANSACTION_OPERATION_LAST_TYPE
} KaimingTransactionOperationType;

/**
 * KaimingTransactionErrorDetails
 * @KAIMING_TRANSACTION_ERROR_DETAILS_NON_FATAL: The operation failure was not fatal
 *
 * The details for #KaimingTransaction::operation-error.
 */
typedef enum {
  KAIMING_TRANSACTION_ERROR_DETAILS_NON_FATAL = 1 << 0,
} KaimingTransactionErrorDetails;

/**
 * KaimingTransactionResult
 * @KAIMING_TRANSACTION_RESULT_NO_CHANGE: The update caused no changes
 *
 * The details for #KaimingTransaction::operation-done.
 */
typedef enum {
  KAIMING_TRANSACTION_RESULT_NO_CHANGE = 1 << 0,
} KaimingTransactionResult;

/**
 * KaimingTransactionRemoteReason
 * @KAIMING_TRANSACTION_REMOTE_GENERIC_REPO: The remote specified in the kaimingref has other apps too
 * @KAIMING_TRANSACTION_REMOTE_RUNTIME_DEPS: The remote has runtimes needed for the app
 *
 * The reason for #KaimingTransaction::add-new-remote.
 */
typedef enum {
  KAIMING_TRANSACTION_REMOTE_GENERIC_REPO,
  KAIMING_TRANSACTION_REMOTE_RUNTIME_DEPS,
} KaimingTransactionRemoteReason;

KAIMING_EXTERN
G_DECLARE_FINAL_TYPE (KaimingTransactionProgress, kaiming_transaction_progress, KAIMING, TRANSACTION_PROGRESS, GObject)

KAIMING_EXTERN
G_DECLARE_FINAL_TYPE (KaimingTransactionOperation, kaiming_transaction_operation, KAIMING, TRANSACTION_OPERATION, GObject)

KAIMING_EXTERN
G_DECLARE_DERIVABLE_TYPE (KaimingTransaction, kaiming_transaction, KAIMING, TRANSACTION, GObject)

struct _KaimingTransactionClass
{
  GObjectClass parent_class;

  void (*new_operation)        (KaimingTransaction          *transaction,
                                KaimingTransactionOperation *operation,
                                KaimingTransactionProgress  *progress);
  void (*operation_done)       (KaimingTransaction          *transaction,
                                KaimingTransactionOperation *operation,
                                const char                  *commit,
                                KaimingTransactionResult     details);
  gboolean (*operation_error)  (KaimingTransaction            *transaction,
                                KaimingTransactionOperation   *operation,
                                const GError                  *error,
                                KaimingTransactionErrorDetails detail);
  int (*choose_remote_for_ref) (KaimingTransaction *transaction,
                                const char         *for_ref,
                                const char         *runtime_ref,
                                const char * const *remotes);
  void (*end_of_lifed)         (KaimingTransaction *transaction,
                                const char         *ref,
                                const char         *reason,
                                const char         *rebase);
  gboolean (*ready)            (KaimingTransaction *transaction);

  gboolean (*add_new_remote) (KaimingTransaction            *transaction,
                              KaimingTransactionRemoteReason reason,
                              const char                    *from_id,
                              const char                    *remote_name,
                              const char                    *url);

  gboolean (*run)            (KaimingTransaction *transaction,
                              GCancellable       *cancellable,
                              GError            **error);
  gboolean (*end_of_lifed_with_rebase) (KaimingTransaction *transaction,
                                        const char         *remote,
                                        const char         *ref,
                                        const char         *reason,
                                        const char         *rebased_to_ref,
                                        const char        **previous_ids);

  gboolean (*webflow_start) (KaimingTransaction *transaction,
                             const char         *remote,
                             const char         *url,
                             GVariant           *options,
                             guint               id);
  void (*webflow_done) (KaimingTransaction *transaction,
                        GVariant           *options,
                        guint               id);

  gboolean (*basic_auth_start) (KaimingTransaction *transaction,
                                const char         *remote,
                                const char         *realm,
                                GVariant           *options,
                                guint               id);
  void (*install_authenticator)   (KaimingTransaction *transaction,
                                   const char         *remote,
                                   const char         *authenticator_ref);

  gboolean (*ready_pre_auth) (KaimingTransaction *transaction);

  gpointer padding[3];
};

KAIMING_EXTERN
KaimingTransaction *kaiming_transaction_new_for_installation (KaimingInstallation *installation,
                                                              GCancellable        *cancellable,
                                                              GError             **error);

KAIMING_EXTERN
void        kaiming_transaction_progress_set_update_frequency (KaimingTransactionProgress *self,
                                                               guint                       update_interval);
KAIMING_EXTERN
char *      kaiming_transaction_progress_get_status (KaimingTransactionProgress *self);
KAIMING_EXTERN
gboolean    kaiming_transaction_progress_get_is_estimating (KaimingTransactionProgress *self);
KAIMING_EXTERN
int         kaiming_transaction_progress_get_progress (KaimingTransactionProgress *self);
KAIMING_EXTERN
guint64     kaiming_transaction_progress_get_bytes_transferred (KaimingTransactionProgress *self);
KAIMING_EXTERN
guint64     kaiming_transaction_progress_get_start_time (KaimingTransactionProgress *self);


KAIMING_EXTERN
KaimingTransactionOperationType kaiming_transaction_operation_get_operation_type (KaimingTransactionOperation *self);
KAIMING_EXTERN
const char *                    kaiming_transaction_operation_get_ref (KaimingTransactionOperation *self);
KAIMING_EXTERN
GPtrArray *                     kaiming_transaction_operation_get_related_to_ops (KaimingTransactionOperation *self);
KAIMING_EXTERN
gboolean                        kaiming_transaction_operation_get_is_skipped (KaimingTransactionOperation *self);
KAIMING_EXTERN
const char *                    kaiming_transaction_operation_get_remote (KaimingTransactionOperation *self);
KAIMING_EXTERN
GFile *                         kaiming_transaction_operation_get_bundle_path (KaimingTransactionOperation *self);
KAIMING_EXTERN
const char *                    kaiming_transaction_operation_get_commit (KaimingTransactionOperation *self);
KAIMING_EXTERN
guint64                         kaiming_transaction_operation_get_download_size (KaimingTransactionOperation *self);
KAIMING_EXTERN
guint64                         kaiming_transaction_operation_get_installed_size (KaimingTransactionOperation *self);
KAIMING_EXTERN
GKeyFile *                      kaiming_transaction_operation_get_metadata (KaimingTransactionOperation *self);
KAIMING_EXTERN
GKeyFile *                      kaiming_transaction_operation_get_old_metadata (KaimingTransactionOperation *self);
KAIMING_EXTERN
const char * const *            kaiming_transaction_operation_get_subpaths (KaimingTransactionOperation *self);
KAIMING_EXTERN
gboolean                        kaiming_transaction_operation_get_requires_authentication (KaimingTransactionOperation *self);
KAIMING_EXTERN
const char *                    kaiming_transaction_operation_type_to_string (KaimingTransactionOperationType kind);

KAIMING_EXTERN
void                kaiming_transaction_set_no_pull (KaimingTransaction *self,
                                                     gboolean            no_pull);
KAIMING_EXTERN
gboolean            kaiming_transaction_get_no_pull (KaimingTransaction *self);
KAIMING_EXTERN
void                kaiming_transaction_set_no_deploy (KaimingTransaction *self,
                                                       gboolean            no_deploy);
KAIMING_EXTERN
gboolean            kaiming_transaction_get_no_deploy (KaimingTransaction *self);
KAIMING_EXTERN
void                kaiming_transaction_set_disable_static_deltas (KaimingTransaction *self,
                                                                   gboolean            disable_static_deltas);
KAIMING_EXTERN
void                kaiming_transaction_set_disable_prune (KaimingTransaction *self,
                                                           gboolean            disable_prune);
KAIMING_EXTERN
void                kaiming_transaction_set_disable_dependencies (KaimingTransaction *self,
                                                                  gboolean            disable_dependencies);
KAIMING_EXTERN
void                kaiming_transaction_set_disable_related (KaimingTransaction *self,
                                                             gboolean            disable_related);
KAIMING_EXTERN
void                kaiming_transaction_set_disable_auto_pin  (KaimingTransaction *self,
                                                               gboolean            disable_pin);
KAIMING_EXTERN
void                kaiming_transaction_set_reinstall (KaimingTransaction *self,
                                                       gboolean            reinstall);
KAIMING_EXTERN
gboolean            kaiming_transaction_get_no_interaction (KaimingTransaction *self);
KAIMING_EXTERN
void                kaiming_transaction_set_no_interaction (KaimingTransaction *self,
                                                            gboolean            no_interaction);
KAIMING_EXTERN
void                kaiming_transaction_set_force_uninstall (KaimingTransaction *self,
                                                             gboolean            force_uninstall);
KAIMING_EXTERN
void                kaiming_transaction_set_default_arch (KaimingTransaction *self,
                                                          const char         *arch);
KAIMING_EXTERN
void                kaiming_transaction_set_parent_window (KaimingTransaction *self,
                                                           const char *parent_window);
KAIMING_EXTERN
const char *        kaiming_transaction_get_parent_window (KaimingTransaction *self);
KAIMING_EXTERN
void                kaiming_transaction_set_include_unused_uninstall_ops (KaimingTransaction *self,
                                                                          gboolean            include_unused_uninstall_ops);
KAIMING_EXTERN
gboolean            kaiming_transaction_get_include_unused_uninstall_ops (KaimingTransaction *self);
KAIMING_EXTERN
void                kaiming_transaction_set_auto_install_sdk (KaimingTransaction *self,
                                                              gboolean            auto_install_sdk);
KAIMING_EXTERN
gboolean            kaiming_transaction_get_auto_install_sdk (KaimingTransaction *self);
KAIMING_EXTERN
void                kaiming_transaction_set_auto_install_debug (KaimingTransaction *self,
                                                                gboolean            auto_install_debug);
KAIMING_EXTERN
gboolean            kaiming_transaction_get_auto_install_debug (KaimingTransaction *self);
KAIMING_EXTERN
void                kaiming_transaction_add_dependency_source (KaimingTransaction  *self,
                                                               KaimingInstallation *installation);
KAIMING_EXTERN
void                kaiming_transaction_add_sideload_repo (KaimingTransaction  *self,
                                                           const char          *path);
KAIMING_EXTERN
void                kaiming_transaction_add_default_dependency_sources (KaimingTransaction *self);
KAIMING_EXTERN
gboolean            kaiming_transaction_run (KaimingTransaction *transaction,
                                             GCancellable       *cancellable,
                                             GError            **error);
KAIMING_EXTERN
KaimingTransactionOperation *kaiming_transaction_get_current_operation (KaimingTransaction *self);
KAIMING_EXTERN
KaimingTransactionOperation *kaiming_transaction_get_operation_for_ref (KaimingTransaction  *self,
                                                                        const char          *remote,
                                                                        const char          *ref,
                                                                        GError             **error);
KAIMING_EXTERN
KaimingInstallation *kaiming_transaction_get_installation (KaimingTransaction *self);
KAIMING_EXTERN
GList *kaiming_transaction_get_operations (KaimingTransaction *self);

KAIMING_EXTERN
void               kaiming_transaction_abort_webflow (KaimingTransaction *self,
                                                      guint               id);
KAIMING_EXTERN
void               kaiming_transaction_complete_basic_auth (KaimingTransaction *self,
                                                            guint id,
                                                            const char *user,
                                                            const char *password,
                                                            GVariant *options);

KAIMING_EXTERN
gboolean            kaiming_transaction_add_install (KaimingTransaction *self,
                                                     const char         *remote,
                                                     const char         *ref,
                                                     const char        **subpaths,
                                                     GError            **error);
KAIMING_EXTERN
gboolean            kaiming_transaction_add_rebase (KaimingTransaction *self,
                                                    const char         *remote,
                                                    const char         *ref,
                                                    const char        **subpaths,
                                                    const char        **previous_ids,
                                                    GError            **error);
KAIMING_EXTERN
gboolean            kaiming_transaction_add_install_bundle (KaimingTransaction *self,
                                                            GFile              *file,
                                                            GBytes             *gpg_data,
                                                            GError            **error);
KAIMING_EXTERN
gboolean            kaiming_transaction_add_install_kaimingref (KaimingTransaction *self,
                                                                GBytes             *kaimingref_data,
                                                                GError            **error);
KAIMING_EXTERN
gboolean            kaiming_transaction_add_update (KaimingTransaction *self,
                                                    const char         *ref,
                                                    const char        **subpaths,
                                                    const char         *commit,
                                                    GError            **error);
KAIMING_EXTERN
gboolean            kaiming_transaction_add_uninstall (KaimingTransaction *self,
                                                       const char         *ref,
                                                       GError            **error);
KAIMING_EXTERN
gboolean            kaiming_transaction_is_empty (KaimingTransaction *self);

G_END_DECLS

#endif /* __KAIMING_TRANSACTION_H__ */
